/* TABS.EXE adds/deletes tab characters (ASCII 9) to/from an ASCII file.
   The destination file must have a different name than the source file.
   The tab interval is adjustable.  Public domain.

   Author:           John Haluska  74000.1106@compuserve.com
   Compiler:         Borland Turbo C V2.0 (Small Model)
   Operating System: MS-DOS    */

#include <stdio.h>  /* printf(), gets(), sscanf(), fopen(), fgets(), fputs(),
                       fclose(), feof() */
#include <string.h> /* strcmp() */
#include <ctype.h>  /* toupper(), tolower() */

char *version = "TABS V1.0";
FILE *fi, *fo;
char st[80], fin[80], fout[80], s1[256], s2[256];
int mode, tabint;

char *strdetab(char *dest, char *src, int tablen);
char *strentab(char *dest, char *src, int tablen);
int file_exist(char *fname);
char *strupc(char *s);
/*--------*/
void main(void)
{
  printf("%s Add or delete tabs from ASCII text file.\n",version);
  printf("(A) Add tabs, (D) Delete tabs, or (Q) Quit: ");
  gets(st);
  switch (tolower(*st)) {
    case 'q': return;
    case 'a': mode = 1;
              break;
    case 'd': mode = 0;
              break;
    default:  printf("Invalid mode.\n");
              return;
  }
  for (;;) {
    printf("Enter tab interval (2 to 20, usually 8) or 0 to quit: ");
    gets(st);
    if (*st == '\0')
      return;
    sscanf(st,"%d",&tabint);
    if (tabint == 0) {
      return;
    }
    if (tabint < 2 || tabint > 20)
      printf("Invalid tab interval.\n");
    else
      break;
  }
  for (;;) {
    printf("Enter ASCII file name to convert: ");
    gets(fin);
    strupc(fin);
    if (!file_exist(fin)) {
      printf("File not found. Try again (Y or N)? ");
      gets(st);
      if (tolower(*st) == 'n') {
        printf("Quit program.\n");
        return;
      }
    }
    else
      break;
  }
  for (;;) {
    printf("Enter destination file name: ");
    gets(fout);
    strupc(fout);
    if (strcmp(fin,fout) == 0) {
      printf("Source and destination files can't have the same name. Try again (Y or N)? ");
      gets(st);
      if (tolower(*st) == 'n') {
        printf("Quit program.\n");
        return;
      }
    }
    else
      break;
  }
  for (;;) {
    if (file_exist(fout)) {
      printf("Destination file already exists. Overwrite (Y or N)? ");
      gets(st);
      if (tolower(*st) == 'n') {
        printf("Quit program.\n");
        return;
      }
      else
        break;
    }
    else
     break;
  }
  fi = fopen(fin,"rt");
  fo = fopen(fout,"wt");
  while (!feof(fi)) {
    fgets(s1,255,fi);
    if (mode == 1)
      strentab(s2,s1,tabint);
    else
      strdetab(s2,s1,tabint);
    if (fputs(s2,fo) == NULL) {
      printf("Can't write all of destination file.\n");
      break;
    }
  }
  fclose(fo);
  fclose(fi);
} /* main */
/*--------*/
/* strdetab() returns string src in string dest with tab character(s) (ASCII 9)
   converted to space(s).  The number of space(s) is from the last non-space
   character to the next tab stop.  Leading space(s) from the tab stop to the
   first non-space character are not converted.  tablen defines the tab stop
   interval.  tablen must be >= 2.
   Example:                          123456781234567812345678
                            si  =   "abc*  abcd*   ab"
       strdetab(so,si,8) returns    "abc       abcd     ab"
   where '*' represents tab char. */

char *strdetab(char *dest, char *src, int tablen)
{
  int j = 0;

  for (;;) {
    *(dest + j) = *src;
    if (*src == '\0')
      break;
    if (*src == '\t')
      do
        *(dest + j++) = ' ';
      while ((j % tablen) != 0);
    else
      j++;
    src++;
  }
  return dest;
} /* strdetab */
/*--------*/
/* strentab() returns string src in string dest with space character(s)
   converted to tab(s) (ASCII 9).  A tab replaces space characters from the
   last non-space character to the next tab stop.  Leading space(s) from the
   tab stop to the first non-space character are not replaced.  tablen defines
   the tab stop interval.  tablen must be >= 2.
   Example:                          123456781234567812345678
                            si  ==  "abc       abcd     ab"
       strentab(so,si,8) returns    "abc*  abcd*   ab"
   where '*' represents tab char. */

char *strentab(char *dest, char *src, int tablen)
{
  int col=0, tcol, j=0;

  for (;;) {
    tcol = col;
    while (*(src + tcol) == ' ') {
      tcol++;
      if (tcol % tablen == 0) {
        *(dest + j++) = '\t';
        col = tcol;
      }
    }
    while (col < tcol) {
      *(dest + j++) = ' ';
      col++;
    }
    if (*(src+tcol) == '\0') {
      *(dest + j) = '\0';
      break;
    }
    *(dest + j++) = *(src + tcol);
    col++;
  }
  return dest;
} /* strentab */
/*--------*/
/* file_exist() returns 1 if the file fname exists, 0 if it doesn't exist. */

int file_exist(char *fname)
{
  FILE *fp;

  if ((fp = fopen(fname,"rt")) == NULL)
    return 0;
  else {
    fclose(fp);
    return 1;
  }
} /* file_exist */
/*--------*/
/* strupc() returns string s with letters a to z changed to all upper case. */

char *strupc(char *s)
{
  char *p;

  p = s;
  while (*p != 0) {
    *p = toupper(*p);
    p++;
  }
  return s;
} /* strupc */
/*--------*/
